-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp - all rights reserved.
-- 
-- For more scripts and sample code, check out 
-- 	http://www.SQLSkills.com
-- For "More than Just Training," see  
-- 	http://www.SolidQualityLearning.com
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

USE Credit
go

-- Create an index and view it's statistics
CREATE INDEX MemberName ON Member(LastName, FirstName, MiddleInitial)
go

DBCC SHOW_STATISTICS(member, MemberName)
go

-- Using Density for LastName
SELECT 10000 * 0.03846154 

-- Using Density for LastName, FirstName Combo
SELECT 10000 * 9.9999997E-5

-- Is it true?
-- FOR LastName
SELECT m.LastName, COUNT(*)
FROM dbo.member AS m
GROUP BY m.LastName

SELECT AVG(Counts.GroupCounts)
FROM (SELECT m.LastName, COUNT(*) AS GroupCounts
		FROM dbo.member AS m
		GROUP BY m.LastName) AS Counts

-- FOR LastName, FirstName
SELECT m.LastName, m.FirstName, COUNT(*)
FROM dbo.member AS m
GROUP BY m.LastName, m.FirstName

SELECT AVG(Counts.GroupCounts)
FROM (SELECT COUNT(*) AS GroupCounts
		FROM dbo.member AS m
		GROUP BY m.LastName, m.FirstName) AS Counts

-- So - what does this tell us about the relationship between LastNames and FirstNames?
exec sp_helpindex member
exec sp_helpstats member -- look at messages window (Object does not have any stats)

-- Update a few rows to later seach on, etc. (not enough to change distribution OR statistics)
UPDATE dbo.Member
	SET LastName = 'Tripp'
	WHERE Member_no IN (678, 9234)
UPDATE dbo.Member
	SET LastName = 'Tripp',
		FirstName = 'Kimberly'
	WHERE Member_no = 1234

-- What would you expect this query to do?
SET STATISTICS IO ON --(turn on showplan with Ctrl+K)
go

SELECT m.LastName, m.FirstName, m.MiddleInitial, m.Phone_no, m.City
FROM dbo.Member AS m
WHERE m.FirstName LIKE 'Kim%'

-- Table Scan (always an option)
-- No Indexes to help find FIRSTNAMES...
-- What about scanning the NC index on LN,FN,MI and then doing bookmarks lookups...
-- seems risky? Would be good if we were guaranteed to only find a VERY small 
-- number of people with firstnames like 'Kim%'?

-- Is this always a good - generally it works well BUT nothing beats REAL statistics? 
-- In fact, SQL Server created statistics on the FirstName column...
exec sp_helpindex member
exec sp_helpstats member

-- Do the existence of "statistics" mean that you MUST create an index... NO but it 
-- is likely that the column is being used in SARGs and/or Join Conditions so it's 
-- something to think about - asking ITW?

-- And can this kind of interpolation go wrong - yes! Especially when the data IS 
-- NOT evenly distributed...

USE Northwind
go

SELECT min(o.OrderDate) AS OldestUnshipped
FROM dbo.Orders AS o WITH (INDEX(0))
WHERE o.ShippedDate IS NULL

SELECT min(o.OrderDate) AS OldestUnshipped
FROM dbo.Orders AS o WITH (INDEX(ShippedDate))
WHERE o.ShippedDate IS NULL

SELECT min(o.OrderDate) AS OldestUnshipped
FROM dbo.Orders AS o
WHERE o.ShippedDate IS NULL

-- Is there anything that would be better... single column indexes (or very narrow indexes)
-- tend to be useful for a narrower number of queries (lame pun intended)

-- A wider index - one that could really tell us what's going on here...

CREATE INDEX OrderDateShipped ON Orders(ShippedDate, OrderDate)
go

SELECT min(o.OrderDate) AS OldestUnshipped
FROM dbo.Orders AS o
WHERE o.ShippedDate IS NULL

SELECT min(o.OrderDate) AS OldestUnshipped
FROM dbo.Orders AS o WITH (INDEX(0))
WHERE o.ShippedDate IS NULL

SELECT min(o.OrderDate) AS OldestUnshipped
FROM dbo.Orders AS o WITH (INDEX(ShippedDate))
WHERE o.ShippedDate IS NULL

SELECT min(o.OrderDate) AS OldestUnshipped
FROM dbo.Orders AS o WITH (Index(OrderDate))
WHERE o.ShippedDate IS NULL

