USE Northwind
go

CREATE VIEW OrderSubTotalsViewNOTOrdered 
AS
SELECT OD.OrderID,
	 SUM(CONVERT(money,(OD.UnitPrice
		*Quantity*(1-Discount)/100))*100) AS SubTotal
FROM [Order Details] AS OD
GROUP BY OD.OrderID
GO

CREATE VIEW OrderSubTotalsViewOrdered 
AS
SELECT TOP 100 PERCENT OD.OrderID,
	 SUM(CONVERT(money,(OD.UnitPrice
		*Quantity*(1-Discount)/100))*100) AS SubTotal
FROM [Order Details] AS OD
GROUP BY OD.OrderID
ORDER BY SubTotal DESC
go

-- No difference between these two... same request, view
-- used consistently...
SELECT * FROM dbo.OrderSubTotalsViewNOTOrdered 
	ORDER BY SubTotal DESC
SELECT * FROM dbo.OrderSubTotalsViewOrdered

-- When the view starts to get used in different ways...
-- Compare the plans (showplan, Ctrl+k) of the two following 
-- queries and you'll see that the view which has the ORDER BY
-- in it has an extra step of ordering the data - which is NOT
-- necessary.
SELECT C.ContactName, Sub.* 
FROM OrderSubTotalsViewOrdered AS Sub
	JOIN Orders AS O ON O.orderid = Sub.Orderid
	JOIN Customers AS c ON c.customerid = o.customerid
WHERE C.City = 'Madrid'
ORDER BY SubTotal DESC -- MUST have this order by because order is not guaranteed
go
SELECT C.ContactName, Sub.* 
FROM OrderSubTotalsViewNOTOrdered AS Sub
	JOIN Orders AS O ON O.orderid = Sub.Orderid
	JOIN Customers AS c ON c.customerid = o.customerid
WHERE C.City = 'Madrid'
ORDER BY SubTotal DESC
go
sp_helptext OrderSubTotalsViewOrdered 
exec sp_helptext OrderSubTotalsViewNOTOrdered 